#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text.RegularExpressions;

namespace HeuristicLab.Grammars {
  public class AttributedGrammar : Grammar {
    public AttributedGrammar(ISymbol startSymbol)
      : base(startSymbol) {
      Debug.Assert(!startSymbol.Equals(EmptySymbol));
      this.StartSymbol = startSymbol;
    }

    private static Regex ruleExpr = new Regex(@"\s*(?<ntSymbol>\w+)\s*(?<formPar>\<\w+\>)+\s*->\s*(?<alternative>\w+\s*(?<actPar>\<\w+\>)+(?:\s+\w+\s*(?<actPar>\<\w+\>)+)*)(?:\s*\|\s*(?<alternative>\w+\s*(?<formPar>\<\w+\>)+(?:\s+\w+\s*(?<formPar>\<\w+\>)+)*))*");
    private static Regex empty = new Regex(@"^\s*$");
    public static Grammar FromString(string gStr) {
      var lines = gStr.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
      lines = lines.Where(l => !empty.IsMatch(l)).ToArray(); // remove empty lines

      // first line is the rule for the start-symbol
      var m = ruleExpr.Match(lines.First());
      var startSymbol = new Symbol(m.Groups["ntSymbol"].Value);
      
      var g = new Grammar(startSymbol);
      foreach (var alt in m.Groups["alternative"].Captures) {
        g.AddProductionRule(startSymbol, new Sequence(alt.ToString().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries).Select(n => new Symbol(n))));
      }
      foreach (var line in lines.Skip(1)) {
        m = ruleExpr.Match(line);
        var ntSymbol = new Symbol(m.Groups["ntSymbol"].Value);
        foreach (var alt in m.Groups["alternative"].Captures) {
          g.AddProductionRule(ntSymbol, new Sequence(alt.ToString().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries).Select(n => new Symbol(n))));
        }
      }

      return g;
    }
  }
}